using System;
using System.Collections.Generic;
using System.Text;
using com.trackplus.webservice.client.trackplusWS;
using com.trackplus.track.ws;

namespace com.trackplus.track.ws.util
{
    public class ItemWrapper
    {
        private ICollection<int> presentFields;
        private WSField[] customFields;

        private WSFieldsConfigBean fieldsConfigBean;
        private WSItemBean itemBean;

        private Dictionary<int,WSOptionBean[]> customFieldsOptions;
        private Dictionary<String,WSFieldSettingsBean> customFieldsSettingsMap;
        private Dictionary<String,String> customFieldsValueMap;
        private Dictionary<String,WSField> customFieldsMap;
        private bool readOnly;

        public ItemWrapper(WSField[] customFields, WSItemContextBean itemCtx)
            :this(customFields, itemCtx.fieldsConfig, itemCtx.item, null){
        }
        public ItemWrapper(WSField[] customFields, WSEditItemContextBean editItemCtx)
            :this(customFields, editItemCtx.fieldsConfig, editItemCtx.item, editItemCtx.form){
            this.readOnly = "true".Equals(editItemCtx.readOnly);
        }
        private ItemWrapper(WSField[] customFields, WSFieldsConfigBean fieldsConfigBean,
                WSItemBean itemBean, WSFormBean form)
        {
            this.customFields = customFields;
            this.fieldsConfigBean = fieldsConfigBean;
            this.itemBean = itemBean;

            initCustomFieldsMap();

            //set present fields
            presentFields = new List<int>();
            if (form == null)
            {
                //set all fields as present
                addSystemFields(presentFields);
                foreach (KeyValuePair<String,WSField> kvp in customFieldsMap)
                {
                    String fieldID = kvp.Key;
                    int id;
                    int.TryParse(fieldID, out id);
                    presentFields.Add(id);
                } 
            }
            else
            {
                presentFields = getFormFields(form);
            }
            //ensure IssueNumer
            if (!presentFields.Contains(SystemFields.ISSUENO))
            {
                presentFields.Add(SystemFields.ISSUENO);
            }
            //ensure IssueType
            if (!presentFields.Contains(SystemFields.ISSUETYPE))
            {
                presentFields.Add(SystemFields.ISSUETYPE);
            }
            //ensure Project
            if (!presentFields.Contains(SystemFields.PROJECT))
            {
                presentFields.Add(SystemFields.PROJECT);
            }
            //ensure Synopsis
            if (!presentFields.Contains(SystemFields.SYNOPSYS))
            {
                presentFields.Add(SystemFields.SYNOPSYS);
            }


            initCustomFieldsOptionsMap();
            initCustomFieldSettingsMap();
            initCustomFieldsValueMap();
        }
        public ICollection<int> getPresentFields()
        {
            return presentFields;
        }
        public Object getValue(String fieldID)
        {
            int id;
            int.TryParse(fieldID, out id);
            return getValue(id);
        }
        private Object getValue(int fieldID)
        {
            switch (fieldID)
            {
                case SystemFields.PROJECT:
                    return itemBean.projectID;
                case SystemFields.ISSUETYPE:
                    return itemBean.issueTypeID;
//                case SystemFields.SUBPROJECT:
//                    return itemBean.subprojectID;
                case SystemFields.STATE:
                    return itemBean.stateID;
                case SystemFields.MANAGER:
                    return itemBean.managerID;
                case SystemFields.RESPONSIBLE:
                    return itemBean.responsibleID;
//                case SystemFields.CLASS:
//                    return itemBean.classID;
                case SystemFields.RELEASENOTICED:
                    return itemBean.releaseNoticedID;
                case SystemFields.RELEASESCHEDULED:
                    return itemBean.releaseScheduledID;
                case SystemFields.PRIORITY:
                    return itemBean.priorityID;
                case SystemFields.SEVERITY:
                    return itemBean.severityID;
                case SystemFields.ISSUENO:
                    return itemBean.objectID;
                case SystemFields.ORIGINATOR:
                    return itemBean.originatorID;
                case SystemFields.CREATEDATE:
                    return itemBean.created;
                case SystemFields.LASTMODIFIEDDATE:
                    return itemBean.lastEdit;
                case SystemFields.SYNOPSYS:
                    return itemBean.synopsis;
                case SystemFields.BUILD:
                    return itemBean.build;
                case SystemFields.STARTDATE:
                    return itemBean.startDate;
                case SystemFields.ENDDATE:
                    return itemBean.endDate;
                case SystemFields.DESCRIPTION:
                    return itemBean.description;
                case SystemFields.SUPERIORWORKITEM:
                    return itemBean.superiorWorkitem;
                case SystemFields.COMMENT:
                    return itemBean.newComment;
            }
            return null;// customFieldsValueMap.get(Integer.toString(fieldID));
        }
        public String getDisplayValue(String fieldID)
        {
            int id;
            int.TryParse(fieldID, out id);
            return getDisplayValue(id);
        }
        protected String getDisplayValue(int fieldID)
        {
            switch (fieldID)
            {
                case SystemFields.PROJECT:
                    return TCLUtil.findDisplayValue(fieldsConfigBean.projectOptions, itemBean.projectID);
                case SystemFields.ISSUETYPE:
                    return TCLUtil.findDisplayValue(fieldsConfigBean.issueTypeOptions, itemBean.issueTypeID);
//                case SystemFields.SUBPROJECT:
//                    return TCLUtil.findDisplayValue(fieldsConfigBean.subprojectOptions, itemBean.subprojectID);
                case SystemFields.STATE:
                    return TCLUtil.findDisplayValue(fieldsConfigBean.stateOptions, itemBean.stateID);
                case SystemFields.MANAGER:
                    return TCLUtil.findDisplayValue(fieldsConfigBean.managerOptions, itemBean.managerID);
                case SystemFields.RESPONSIBLE:
                    return TCLUtil.findDisplayValue(fieldsConfigBean.responsibleOptions, itemBean.responsibleID);
//                case SystemFields.CLASS:
//                    return TCLUtil.findDisplayValue(fieldsConfigBean.classOptions, itemBean.classID);
                case SystemFields.RELEASENOTICED:
                    return TCLUtil.findDisplayValue(fieldsConfigBean.releaseNoticedOptions, itemBean.releaseNoticedID);
                case SystemFields.RELEASESCHEDULED:
                    return TCLUtil.findDisplayValue(fieldsConfigBean.releaseScheduledOptions, itemBean.releaseScheduledID);
                case SystemFields.PRIORITY:
                    return TCLUtil.findDisplayValue(fieldsConfigBean.priorityOptions, itemBean.priorityID);
                case SystemFields.SEVERITY:
                    return TCLUtil.findDisplayValue(fieldsConfigBean.severityOptions, itemBean.severityID);
                case SystemFields.ISSUENO:
                    return itemBean.objectID;
                case SystemFields.ORIGINATOR:
                    return TCLUtil.findDisplayValue(fieldsConfigBean.originatorOptions, itemBean.originatorID);
                case SystemFields.CREATEDATE:
                    //TODO format date
                    return itemBean.created;
                case SystemFields.LASTMODIFIEDDATE:
                    return itemBean.lastEdit;
                case SystemFields.SYNOPSYS:
                    return itemBean.synopsis;
                case SystemFields.BUILD:
                    return itemBean.build;
                case SystemFields.STARTDATE:
                    return itemBean.startDate;
                case SystemFields.ENDDATE:
                    return itemBean.endDate;
                case SystemFields.DESCRIPTION:
                    return itemBean.description;
                case SystemFields.SUPERIORWORKITEM:
                    return itemBean.superiorWorkitem;
                case SystemFields.COMMENT:
                    return itemBean.newComment;
            }
            return null;// (String)customFieldsValueMap.get(Integer.toString(fieldID));
        }

        public void setAttributeValue(int fieldID, String value)
        {
            switch (fieldID)
            {
                case SystemFields.PROJECT:
                    itemBean.projectID=value;
                    return;
                case SystemFields.ISSUETYPE:
                    itemBean.issueTypeID=value;
                    return;
//                case SystemFields.SUBPROJECT:
//                    itemBean.subprojectID=value;
//                    return;
                case SystemFields.STATE:
                    itemBean.stateID=value;
                    return;
                case SystemFields.MANAGER:
                    itemBean.managerID=value;
                    return;
                case SystemFields.RESPONSIBLE:
                    itemBean.responsibleID=value;
                    return;
//                case SystemFields.CLASS:
//                    itemBean.classID=value;
//                    return;
                case SystemFields.RELEASENOTICED:
                    itemBean.releaseNoticedID=value;
                    return;
                case SystemFields.RELEASESCHEDULED:
                    itemBean.releaseScheduledID=value;
                    return;
                case SystemFields.PRIORITY:
                    itemBean.priorityID=value;
                    return;
                case SystemFields.SEVERITY:
                    itemBean.severityID=value;
                    return;
                case SystemFields.ISSUENO:
                    itemBean.objectID=value;
                    return;
                case SystemFields.ORIGINATOR:
                    itemBean.originatorID=value;
                    return;
                case SystemFields.CREATEDATE:
                    itemBean.created=value;
                    return;
                case SystemFields.LASTMODIFIEDDATE:
                    itemBean.lastEdit=value;
                    return;
                case SystemFields.SYNOPSYS:
                    itemBean.synopsis=value;
                    return;
                case SystemFields.BUILD:
                    itemBean.build=value;
                    return;
                case SystemFields.STARTDATE:
                    itemBean.startDate=value;
                    return;
                case SystemFields.ENDDATE:
                    itemBean.endDate=value;
                    return;
                case SystemFields.DESCRIPTION:
                    itemBean.description=value;
                    return;
                case SystemFields.SUPERIORWORKITEM:
                    itemBean.superiorWorkitem=value;
                    return;
                case SystemFields.COMMENT:
                    itemBean.newComment=value;
                    return;
            }
            customFieldsValueMap.Add(fieldID + "", value);
            updateCustomFieldsValue();
        }
        private void updateCustomFieldsValue()
        {
            WSCustomFieldValue[] customFieldValues = new WSCustomFieldValue[customFieldsValueMap.Count];
            IEnumerator<String> it = customFieldsValueMap.Keys.GetEnumerator();
            int i = 0;
            WSCustomFieldValue customFieldValue;
            while (it.MoveNext())
            {
                customFieldValue = new WSCustomFieldValue();
                String fieldID = (String)it.Current;
                customFieldValue.fieldId=fieldID;
                customFieldValue.value = null;// ((String)customFieldsValueMap.get(fieldID));
                customFieldValues[i++] = customFieldValue;
            }
            itemBean.customFieldValues=customFieldValues;
        }
        public String getLabel(String fieldID)
        {
            int id;
            int.TryParse(fieldID, out id);
            return getLabel(id);
        }
        private String getLabel(int fieldID)
        {
            switch (fieldID)
            {
                case SystemFields.PROJECT:
                    return fieldsConfigBean.projectSettings.label;
                case SystemFields.ISSUETYPE:
                    return fieldsConfigBean.issueTypeSettings.label;
//                case SystemFields.SUBPROJECT:
//                    return fieldsConfigBean.subprojectSettings.label;
                case SystemFields.STATE:
                    return fieldsConfigBean.stateSettings.label;
                case SystemFields.MANAGER:
                    return fieldsConfigBean.managerSettings.label;
                case SystemFields.RESPONSIBLE:
                    return fieldsConfigBean.responsibleSettings.label;
//                case SystemFields.CLASS:
//                    return fieldsConfigBean.classSettings.label;
                case SystemFields.RELEASENOTICED:
                    return fieldsConfigBean.releaseNoticedSettings.label;
                case SystemFields.RELEASESCHEDULED:
                    return fieldsConfigBean.releaseScheduledSettings.label;
                case SystemFields.PRIORITY:
                    return fieldsConfigBean.prioritySettings.label;
                case SystemFields.SEVERITY:
                    return fieldsConfigBean.severitySettings.label;
                case SystemFields.ISSUENO:
                    //this field is added forced 
                    //if IssueNoSettings is null then this field is not present on screen so we put a hardcoded label
                    return fieldsConfigBean.issueNoSettings == null ? "IssueNo" : fieldsConfigBean.issueNoSettings.label;
                case SystemFields.ORIGINATOR:
                    return fieldsConfigBean.originatorSettings.label;
                case SystemFields.CREATEDATE:
                    return fieldsConfigBean.createdSettings.label;
                case SystemFields.LASTMODIFIEDDATE:
                    //this field is added forced(
                    //if LastEditSettings is null then this field is not present on screen so we put a hardcoded label
                    return fieldsConfigBean.lastEditSettings == null ? "Last edit" : fieldsConfigBean.lastEditSettings.label;
                case SystemFields.SYNOPSYS:
                    return fieldsConfigBean.synopsysSettings.label;
                case SystemFields.BUILD:
                    return fieldsConfigBean.buildSettings.label;
                case SystemFields.STARTDATE:
                    return fieldsConfigBean.startDateSettings.label;
                case SystemFields.ENDDATE:
                    return fieldsConfigBean.endDateSettings.label;
                case SystemFields.DESCRIPTION:
                    return fieldsConfigBean.descriptionSettings.label;
                case SystemFields.SUPERIORWORKITEM:
                    return fieldsConfigBean.superiorWorkitemSettings.label;
                case SystemFields.COMMENT:
                    //this field is added forced 
                    //if CommentSettings is null then this field is not present on screen so we put a hardcoded label
                    return fieldsConfigBean.commentSettings == null ? "Comment" : fieldsConfigBean.commentSettings.label;
            }
            WSFieldSettingsBean fs;
            customFieldsSettingsMap.TryGetValue(fieldID + "",out fs);
            if (fs != null)
            {
                return fs.label;
            }
            else
            {
                WSField field;
                customFieldsMap.TryGetValue(fieldID + "",out field);
                if (field != null)
                {
                    return field.label;
                }
            }
            return "field:" + fieldID;
        }

        public bool isReadOnly(String fieldID)
        {
            int id;
            int.TryParse(fieldID, out id);
            return isReadOnly(id);
        }
        public bool isReadOnly(int id)
        {
            if (readOnly)
            {
                return true;
            }
            switch (id)
            {
                case SystemFields.ISSUENO:
                case SystemFields.ORIGINATOR:
                case SystemFields.PROJECT:
                case SystemFields.ISSUETYPE:
                case SystemFields.CREATEDATE:
                case SystemFields.LASTMODIFIEDDATE:
                    return true;
//                case SystemFields.SUBPROJECT:
//                    return "true".Equals(fieldsConfigBean.subprojectSettings.readOnly);
                case SystemFields.STATE:
                    return "true".Equals(fieldsConfigBean.stateSettings.readOnly);
                case SystemFields.MANAGER:
                    return "true".Equals(fieldsConfigBean.managerSettings.readOnly);
                case SystemFields.RESPONSIBLE:
                    return "true".Equals(fieldsConfigBean.responsibleSettings.readOnly);
//                case SystemFields.CLASS:
//                    return "true".Equals(fieldsConfigBean.classSettings.readOnly);
                case SystemFields.RELEASENOTICED:
                    return "true".Equals(fieldsConfigBean.releaseNoticedSettings.readOnly);
                case SystemFields.RELEASESCHEDULED:
                    return "true".Equals(fieldsConfigBean.releaseScheduledSettings.readOnly);
                case SystemFields.PRIORITY:
                    return "true".Equals(fieldsConfigBean.prioritySettings.readOnly);
                case SystemFields.SEVERITY:
                    return "true".Equals(fieldsConfigBean.severitySettings.readOnly);
                case SystemFields.SYNOPSYS:
                    return "true".Equals(fieldsConfigBean.synopsysSettings.readOnly);
                case SystemFields.BUILD:
                    return "true".Equals(fieldsConfigBean.buildSettings.readOnly);
                case SystemFields.STARTDATE:
                    return "true".Equals(fieldsConfigBean.startDateSettings.readOnly);
                case SystemFields.ENDDATE:
                    return "true".Equals(fieldsConfigBean.endDateSettings.readOnly);
                case SystemFields.DESCRIPTION:
                    return "true".Equals(fieldsConfigBean.descriptionSettings.readOnly);
                case SystemFields.SUPERIORWORKITEM:
                    return "true".Equals(fieldsConfigBean.superiorWorkitemSettings.readOnly);
                case SystemFields.COMMENT:
                    //hardcoded added field
                    if (fieldsConfigBean.commentSettings == null)
                    {
                        return false;
                    }
                    return "true".Equals(fieldsConfigBean.commentSettings.readOnly);
                default:
                    break;
            }
            WSFieldSettingsBean fs;
            customFieldsSettingsMap.TryGetValue(id + "",out fs);
            if (fs != null)
            {
                return "true".Equals(fs.readOnly);
            }
            return false;
        }

        public WSOptionBean[] getOptions(String fieldID)
        {
            int id;
            int.TryParse(fieldID,out id);
            switch (id)
            {
                case SystemFields.ISSUETYPE:
                    return transformOption(fieldsConfigBean.issueTypeOptions);
                case SystemFields.PRIORITY:
                    return transformOption(fieldsConfigBean.priorityOptions);
                /*case SystemFields.RELEASENOTICED:
                    return transformOption(fieldsConfigBean.releaseNoticedOptions);
                case SystemFields.RELEASESCHEDULED:
                    return transformOption(fieldsConfigBean.releaseScheduledOptions);*/
                case SystemFields.MANAGER:
                    return transformOption(fieldsConfigBean.managerOptions);
                case SystemFields.RESPONSIBLE:
                    return transformOption(fieldsConfigBean.responsibleOptions);
                case SystemFields.SEVERITY:
                    return transformOption(fieldsConfigBean.severityOptions);
                case SystemFields.STATE:
                    return transformOption(fieldsConfigBean.stateOptions);
//                case SystemFields.SUBPROJECT:
//                    return fieldsConfigBean.subprojectOptions;
//                case SystemFields.CLASS:
//                    return fieldsConfigBean.classOptions;
                case SystemFields.ORIGINATOR:
                    return transformOption(fieldsConfigBean.originatorOptions);
//                case SystemFields.PROJECT:
//                    return fieldsConfigBean.projectOptions;
            }
            WSOptionBean[] result;
            customFieldsOptions.TryGetValue(id,out result);
            return result;
        }

        public WSItemBean getItemBean()
        {
            return itemBean;
        }

        public WSFieldsConfigBean getFieldsConfigBean()
        {
            return fieldsConfigBean;
        }

        public static ICollection<int> getFormFields(WSFormBean formBean)
        {
            ICollection<int> result = new List<int>();
            WSFormTabBean[] tabs = formBean.tabs;
            for (int i = 0; i < tabs.Length; i++)
            {
                WSFormPanelBean[] panels = tabs[i].panels;
                if (panels != null)
                {
                    for (int j = 0; j < panels.Length; j++)
                    {
                        WSFormFieldBean[] formFields = panels[j].fields;
                        if (formFields != null)
                        {
                            for (int k = 0; k < formFields.Length; k++)
                            {
                                if (formFields[k] != null)
                                {
                                    int id;
                                    int.TryParse(formFields[k].fieldID, out id);
                                    if (!result.Contains(id))
                                    {
                                        result.Add(id);
                                    }
                                }
                            }
                        }
                    }
                }
            }
            return result;
        }

        private void initCustomFieldsMap()
        {
            customFieldsMap = new Dictionary<String, WSField>();
            if (customFields != null)
            {
                for (int i = 0; i < customFields.Length; i++)
                {
                    customFieldsMap.Add(customFields[i].id, customFields[i]);
                }
            }
        }

        private void addSystemFields(ICollection<int> fields)
        {
            fields.Add(SystemFields.ACCESSLEVEL);
            fields.Add(SystemFields.BUILD);
            fields.Add(SystemFields.CHANGEDBY);
            fields.Add(SystemFields.CLASS);
            fields.Add(SystemFields.COMMENT);
            fields.Add(SystemFields.CREATEDATE);
            fields.Add(SystemFields.DESCRIPTION);
            fields.Add(SystemFields.ENDDATE);
            fields.Add(SystemFields.ISSUENO);
            fields.Add(SystemFields.ISSUETYPE);
            fields.Add(SystemFields.LASTMODIFIEDDATE);
            fields.Add(SystemFields.MANAGER);
            fields.Add(SystemFields.ORIGINATOR);
            fields.Add(SystemFields.PRIORITY);
            fields.Add(SystemFields.PROJECT);
            fields.Add(SystemFields.RELEASENOTICED);
            fields.Add(SystemFields.RELEASESCHEDULED);
            fields.Add(SystemFields.RESPONSIBLE);
            fields.Add(SystemFields.SEVERITY);
            fields.Add(SystemFields.STARTDATE);
            fields.Add(SystemFields.STATE);
            fields.Add(SystemFields.SUBPROJECT);
            fields.Add(SystemFields.SUPERIORWORKITEM);
            fields.Add(SystemFields.SYNOPSYS);

        }

        private void initCustomFieldsOptionsMap()
        {
            customFieldsOptions = new Dictionary<int,WSOptionBean[]>();
            WSFieldOptions[] dds = fieldsConfigBean.customFieldOptions;
            if (dds != null)
            {
                for (int i = 0; i < dds.Length; i++)
                {
                    if (dds[i].fieldID != null && dds[i].options != null)
                    {
                        int id;
                        int.TryParse(dds[i].fieldID, out id);
                        customFieldsOptions.Add(id, dds[i].options);
                    }
                }
            }
        }
        private void initCustomFieldSettingsMap()
        {
            customFieldsSettingsMap = new Dictionary<String,WSFieldSettingsBean>();
            WSFieldSettingsBean[] fs = fieldsConfigBean.customFieldSettings;
            if (fs != null)
            {
                for (int i = 0; i < fs.Length; i++)
                {
                    if (fs[0] != null)
                    {
                        customFieldsSettingsMap.Add(fs[i].fieldID, fs[i]);
                    }
                }
            }
        }
        private void initCustomFieldsValueMap()
        {
            customFieldsValueMap = new Dictionary<String, String>();
            WSCustomFieldValue[] customFieldValues = itemBean.customFieldValues;
            if (customFieldValues != null)
            {
                for (int i = 0; i < customFieldValues.Length; i++)
                {
                    customFieldsValueMap.Add(customFieldValues[i].fieldId, customFieldValues[i].value);
                }
            }
        }
        private WSOptionBean[] transformOption(WSStateOptionBean[] stateOptions)
        {
            if (stateOptions == null)
            {
                return null;
            }
            WSOptionBean[] options = new WSOptionBean[stateOptions.Length];
            for (int i = 0; i < stateOptions.Length; i++)
            {
                options[i] = new WSOptionBean();
                options[i].label=stateOptions[i].label;
                options[i].objectID=stateOptions[i].objectID;
            }
            return options;
        }

        private WSOptionBean[] transformOption(WSSortedOptionBean[] sortedOptionBeans)
        {
            if (sortedOptionBeans == null)
            {
                return null;
            }
            WSOptionBean[] options = new WSOptionBean[sortedOptionBeans.Length];
            for (int i = 0; i < sortedOptionBeans.Length; i++)
            {
                options[i] = new WSOptionBean();
                options[i].label=sortedOptionBeans[i].label;
                options[i].objectID=sortedOptionBeans[i].objectID;
            }
            return options;
        }
        private WSOptionBean[] transformOption(WSPersonBean[] personsBeans)
        {
            if (personsBeans == null)
            {
                return null;
            }
            WSOptionBean[] options = new WSOptionBean[personsBeans.Length];
            for (int i = 0; i < personsBeans.Length; i++)
            {
                options[i] = new WSOptionBean();
                options[i].label=personsBeans[i].fullName;
                options[i].objectID=personsBeans[i].personID;
            }
            return options;
        }
    }
}
