using System;
using System.Collections.Generic;
using System.Text;
using System.Globalization;
using com.trackplus.webservice.client.trackplusWS;
using com.trackplus.track.ws.tcl;

namespace com.trackplus.track.ws.util
{
    public class TCLUtil
    {
       
     /**
	 * Format a date in ISO date format yyyy-MM-dd
	 * @param date
	 * @return
	 */
	public static String formatISODate(DateTime? date) {
		if(!date.HasValue){
		 	return null;
		}
        return String.Format("{0:"+TCLFacade.ISO_DATE_FORMAT+"}", date);
	}
	
	/**
	 * Format a date in ISO date-time format yyyy-MM-dd H:mm:ss.S
	 * @param date
	 * @return
	 */
	public static String formatISODateTime(DateTime? date) {
		if(!date.HasValue){
			return null;
		}
        return String.Format("{0:" + TCLFacade.ISO_DATE_TIME_FORMAT+ "}", date);
	}
	
	/**
	 * Parse a date from ISO dateTime format yyyy-MM-dd H:mm:ss.S
	 * @param dateTime
	 * @return
	 * @throws ParseException the dateTime pattern is invalid
	 */
	public static DateTime? parseDateTimeIso(String dateTime) {
		if(dateTime==null||dateTime.Length==0){
            return null;
		}
        return DateTime.Parse(dateTime,new CultureInfo("en-US"));
	}
	
	/**
	 * Parse a date from ISO date format yyyy-MM-dd
	 * @param date
	 * @return
	 * @throws ParseException
	 */
	public static DateTime? parseDateIso(String date) {
		if(date==null||date.Length==0){
            return null;
		}
        return DateTime.Parse(date,new CultureInfo("en-US"));
	}
	
	/**
	 * Find display value for option
	 * @param options
	 * @param objectID
	 * @return
	 */
	public static String findDisplayValue(WSOptionBean[] options,String objectID){
		if(objectID==null||objectID.Length==0){
			return "";
		}
		for (int i = 0; i < options.Length; i++) {
			if(options[i].objectID.Equals(objectID)){
				return options[i].label;
			}
		}
		return null;
	}

    public static String findDisplayValue(WSTreeNode[] options, String objectID)
    {
        if (objectID == null || objectID.Length == 0 || options == null)
        {
            return null;
        }
        for (int i = 0; i < options.Length; i++)
        {
            if (options[i].id.Equals(objectID))
            {
                return options[i].label;
            }
            else
            {
                if (options[i].children != null)
                {
                    String displayChildren = findDisplayValue(options[i].children, objectID);
                    if (displayChildren != null)
                    {
                        return displayChildren;
                    }
                }
            }
        }
        return null;
    }
	
	/**
	 * Find full name for persons
	 * @param persons
	 * @param personID
	 * @return
	 */
	public static String findDisplayValue(WSPersonBean[] persons,String personID){
		if(personID==null||personID.Length==0){
			return "";
		}
		for (int i = 0; i < persons.Length; i++) {
			if(persons[i].personID.Equals(personID)){
				return persons[i].fullName;
			}
		}
		return null;
	}
	
	/**
	 * Find display value for sorted option
	 * @param sortedOptions
	 * @param optionID
	 * @return
	 */
	public static String findDisplayValue(WSSortedOptionBean[] sortedOptions,String optionID) {
		if(optionID==null||optionID.Length==0){
			return "";
		}
		for (int i = 0; i < sortedOptions.Length; i++) {
			if(sortedOptions[i].objectID.Equals(optionID)){
				return sortedOptions[i].label;
			}
		}
		return null;
	}
	/**
	 * Find the display value for state
	 * @param stateOptions
	 * @param stateID
	 * @return
	 */
	public static String findDisplayValue(WSStateOptionBean[] stateOptions,String stateID) {
		if(stateID==null||stateID.Length==0){
			return "";
		}
		for (int i = 0; i < stateOptions.Length; i++) {
			if(stateOptions[i].objectID.Equals(stateID)){
				return stateOptions[i].label;
			}
		}
		return null;
	}
    }
}
  