using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using com.trackplus.webservice.client.trackplusWS;
using com.trackplus.track.ws;
using com.trackplus.track.ws.tcl;
using com.trackplus.track.ws.util;

namespace com.trackplus.track.ws
{
    public class AttachmentSample : LoginSample
    {
        public AttachmentSample()
            : base()
        {
        }
        public AttachmentSample(TCLFacade connector)
            : base(connector)
        {
        }

        public WSAttachmentBean addAttachment(String itemId, String attachName, String attachDescription, String file)
        {
            Console.WriteLine("->AttachmentSample.addAttachment()");
            WSAttachmentBean attach = new WSAttachmentBean();
            attach.attachName = attachName;
            attach.description = attachDescription;
            attach.contentType = "text/html";
            attach.itemKey = itemId;
            //attach.size=file.Length()+"";
            attach.fileName = file;
            WSAttachmentBean attachResult = null;
            try
            {
                attachResult = connector.saveAttachment(attach);
                Console.WriteLine("Succesfully added the attachments:"+attachResult.attachName+" on item:"+itemId);
            }
            catch (TCLFacadeException e)
            {
                Console.WriteLine("Can't add attachments");
                handleTCLFacadeException(e);
            }
            catch (Exception e)
            {
                //connection problems like server is not started, connection time-out,...
                Console.WriteLine("RemoteException.message:" + e.Message);
            }
            Console.WriteLine("<-AttachmentSample.addAttachemt()");
            return attachResult;
        }

        public void printAttachments(WSAttachmentBean[] attachments)
        {
            if (attachments == null || attachments.Length == 0)
            {
                Console.WriteLine("No attachments\n\n");
            }
            else
            {
                Console.WriteLine("\n" + attachments.Length + " attachments found:");
                for (int i = 0; i < attachments.Length; i++)
                {
                    Console.WriteLine("\tID=" + attachments[i].attachKey);
                    Console.WriteLine("\tDate=" + attachments[i].createDate);
                    Console.WriteLine("\tOriginator=" + attachments[i].originator.personID + ":" + attachments[i].originator.fullName);
                    Console.WriteLine("\tName=" + attachments[i].attachName);
                    Console.WriteLine("\tDescription=" + attachments[i].description);
                    Console.WriteLine();
                }
                Console.WriteLine("\n");
            }
        }
        public WSAttachmentBean[] getItemAttachments(String itemId)
        {
            WSAttachmentBean[] attachments = null;
            try
            {
                WSItemBean item = connector.getItem(itemId);
                attachments = item.attachments;
            }
            catch (TCLFacadeException e)
            {
                handleTCLFacadeException(e);
            }
            catch (Exception e)
            {
                //connection problems like server is not started, connection time-out,...
                Console.WriteLine("RemoteException.message:" + e.Message);
            }
            return attachments;
        }

        public void addAttachmentBynaryData(String itemId, String attachName, String attachDescription, byte[] binaryData)
        {
            Console.WriteLine("->AttachmentSample.addAttachmentInputStream()");
            WSAttachmentBean attach = new WSAttachmentBean();
            attach.attachName = attachName;
            attach.description = attachDescription;
            attach.contentType = "img/png";
            attach.itemKey = itemId;
            
            try
            {
                WSAttachmentBean attachResult=connector.saveAttachment(attach, binaryData);
                Console.WriteLine("Succesfully add the attachment as binary data :"+attachResult.attachName+" on item:"+itemId);
            }
            catch (TCLFacadeException e)
            {
                Console.WriteLine("Can't add attachments");
                handleTCLFacadeException(e);
            }

            Console.WriteLine("<-AttachmentSample.addAttachmentInputStream()");
        }

        public void deleteAttachemnt(WSAttachmentBean attach)
        {
            Console.WriteLine("->AttachmentSample.deleteAttachemnt()");
            try
            {
                connector.deleteAttachment(attach);
                Console.WriteLine("Succesfully added the attachment as stream :" + attach.attachName + " on item:" + attach.itemKey);
            }
            catch (TCLFacadeException e)
            {
                handleTCLFacadeException(e);
            }
            catch (Exception e)
            {
                //connection problems like server is not started, connection time-out,...
                Console.WriteLine("RemoteException.message:" + e.Message);
            }
            Console.WriteLine("<-AttachmentSample.deleteAttachemnt()");
        }

        public void downloadAttachment(WSAttachmentBean attach)
        {
            Console.WriteLine("->AttachmentSample.downloadAttachment()");
            byte[] binaryData = null;
            try
            {
                binaryData = connector.downloadAttachment(attach);
            }
            catch (TCLFacadeException e)
            {
                handleTCLFacadeException(e);
            }
            catch (Exception e)
            {
                //connection problems like server is not started, connection time-out,...
                Console.WriteLine("RemoteException.message:" + e.Message);
            }
            Directory.CreateDirectory("etc/download/");
            BinaryWriter binaryWriter = new BinaryWriter(File.Open("etc/download/" + attach.attachName, FileMode.Create));
            binaryWriter.Write(binaryData);
            binaryWriter.Close();
            Console.WriteLine("Succesfully download attachment as:"+"etc/download/" + attach.attachName);

            //File downloaded=new File(fileName);
            //downloaded.delete();
            Console.WriteLine("<-AttachmentSample.downloadAttachment()");
        }

        public String createItem()
        {
            //to create an item we need an project and issueType
            String projectID = null;
            String issueTypeID = null;
            //getting the project for the user 
            try
            {
                WSLabelValueBean[] projects = connector.getProjects();
                if (projects == null || projects.Length == 0)
                {
                    Console.WriteLine("No projects found for user:" + user);
                    return null;
                }
                projectID = projects[0].value;
                WSLabelValueBean[] issueTypes = connector.getIssueTypes(projectID);
                if (issueTypes == null || issueTypes.Length == 0)
                {
                    Console.WriteLine("No issueTypes found for user:" + user);
                    return null;
                }
                issueTypeID = issueTypes[0].value;
            }
            catch (TCLFacadeException e)
            {
                Console.WriteLine("Error obtaining the project and issue type");
                handleTCLFacadeException(e);
                return null;
            }
            catch (Exception e)
            {
                //connection problems like server is not started, connection time-out,...
                Console.WriteLine("RemoteException.message:" + e.Message);
            }
            WSEditItemContextBean editItemCtx = null;
            try
            {
                editItemCtx = connector.getNewItemContext(projectID, issueTypeID);
            }
            catch (TCLFacadeException e)
            {
                Console.WriteLine("Error obtaining the newItemContext");
                handleTCLFacadeException(e);
            }
            catch (Exception e)
            {
                //connection problems like server is not started, connection time-out,...
                Console.WriteLine("RemoteException.message:" + e.Message);
            }
            WSItemBean item = editItemCtx.item;

            //the fieldsConfig contains settings(label, tooltip, readOnly) and options only for fields 
            //present on the form configured for NEW item in given project and issueType
            //and if we change a field that is not present on the form will do not have effect.
            WSFieldsConfigBean cfg = editItemCtx.fieldsConfig;

            //set some values
            item.synopsis = "WebService item";
            item.description = "This item was creared from webservice";

            //save the item
            String itemID = null;
            try
            {
                itemID = connector.saveItem(item);
                Console.WriteLine("Item saved succesfuly: The new itemID is:" + itemID);
            }
            catch (TCLFacadeException e)
            {
                Console.WriteLine("Failed to save the item");
                handleTCLFacadeException(e);
            }
            catch (Exception e)
            {
                //connection problems like server is not started, connection time-out,...
                Console.WriteLine("RemoteException.message:" + e.Message);
            }
            return itemID;
        }
        
        public static void runAttachmentSample(String[] args)
        {
            AttachmentSample attachmentSample = new AttachmentSample();
            attachmentSample.login();

            String itemID = attachmentSample.createItem();

            if (itemID != null)
            {
                //addAttachent
                String attachName = "dummyHtml.html";
                String attachDescription = "some attach description";
                String file = "etc/dummyHtml.html";
                attachmentSample.addAttachment(itemID, attachName, attachDescription, file);

                //see the attachment
                WSAttachmentBean[] attachments = attachmentSample.getItemAttachments(itemID);
                attachmentSample.printAttachments(attachments);

                //add an image attachment
                String attachName2 = "photo.png";
                String attachDescription2 = "some attach description2";
                String file2 = "etc/photo.png";
                
                FileStream fs = new FileStream(file2, FileMode.OpenOrCreate, FileAccess.Read);
                byte[] binaryData = new byte[fs.Length];
                fs.Read(binaryData, 0, System.Convert.ToInt32(fs.Length));
                fs.Close();

                attachmentSample.addAttachmentBynaryData(itemID, attachName2, attachDescription2, binaryData);

                //see the attachment
                attachments = attachmentSample.getItemAttachments(itemID);
                attachmentSample.printAttachments(attachments);

                //download attachment
                //see etc/download directory
                attachmentSample.downloadAttachment(attachments[0]);

                //delete the attachment
                attachmentSample.deleteAttachemnt(attachments[0]);

                //see the attachments
                attachments = attachmentSample.getItemAttachments(itemID);
                attachmentSample.printAttachments(attachments);
            }
            else
            {
                Console.WriteLine("Can't create item");
            }
        }
    }
    
}
