/*
 * This file is part of the Track+ application, a
 * tool for project and change management.
 *
 * Copyright (C) 2014 Steinbeis GmbH & Co. KG
 *
 * Use and distribution of this software is governed by the
 * Track+ license conditions.
 *
 * $Id: FormSample.cs 1344 2015-09-07 20:39:21Z friedric $
 */
using System;
using System.Collections.Generic;
using System.Text;

using com.trackplus.webservice.client.trackplusWS;
using com.trackplus.track.ws;
using com.trackplus.track.ws.tcl;
using com.trackplus.track.ws.util;

namespace com.trackplus.track.ws
{
    /**
     * <p>
     * In Track+ you can define any number of forms (input masks) and assign
     * them to actions like edit, change status, and so on, depending on project
     * and issue type.</p>
     * <p>
     * This class demonstrates how to access the form definition information
     * via the web service interface.
     * </p>
     * <p>
     * The class uses the {@link LoginSample} class to perform
     * the login. You need to set the URL and the user name and password in that
     * class before you compile and run this class here.</p>
     * <p>
     * This example is a C# console application. You can run it directly by
     * calling its <code>runFormSample()</code> method.</p>
     */
    public class FormSample : LoginSample
    {
        public FormSample()
            : base()
        {
        }
        public FormSample(TCLFacade connector)
            : base(connector)
        {
        }
        public void getEditForm()
        {
            Console.WriteLine("\n->FormSample.getEditForm()");
            String projectID = null;
            String issueTypeID = null;
            //getting the project for the user 
            try {
                WSLabelValueBean[] projects= connector.getProjects();
                if(projects==null||projects.Length==0){
                    Console.WriteLine("No projects found for user:"+user);
                    return;
                }
                projectID=projects[0].value;
                WSLabelValueBean[] issueTypes=connector.getIssueTypes(projectID);
                if(issueTypes==null||issueTypes.Length==0){
                    Console.WriteLine("No issueTypes found for user:"+user);
                    return;
                }
                issueTypeID=issueTypes[0].value;
             } catch (TCLFacadeException e) {
                Console.WriteLine("Error obtaining the project and issue type");
                handleTCLFacadeException(e);
                return;
            }
            try
            {
                //if projectID==null and issueTypeID==null we obtain the default edit form
                WSFormBean formEdit = connector.getEditItemForm(projectID, issueTypeID);
                Console.WriteLine("Succesfuly obtain the form for project=" + projectID + " and issueTypes=" + issueTypeID);
                printForm(formEdit);
            }
            catch (TCLFacadeException e)
            {
                Console.WriteLine("Error obtaining the form");
                handleTCLFacadeException(e);
            }
            catch (Exception e)
            {
                //connection problems like server is not started, connection time-out,...
                Console.WriteLine("RemoteException.message:" + e.Message);
            }
            Console.WriteLine("<-FormSample.getEditForm()\n");
        }
        public static void printForm(WSFormBean formBean)
        {
            WSFormTabBean[] tabs = formBean.tabs;
            for (int i = 0; i < tabs.Length; i++)
            {
                WSFormTabBean tab = tabs[i];
                printFormTab(tab);
            }
        }
        private static void printFormTab(WSFormTabBean tab)
        {
            Console.WriteLine("_________________");
            Console.WriteLine("|TAB: " + tab.label + "|");
            Console.WriteLine("_________________");
            Console.WriteLine("");
            WSFormPanelBean[] panels = tab.panels;
            for (int i = 0; i < panels.Length; i++)
            {
                WSFormPanelBean panel = panels[i];
                printFormPanel(panel);
            }
            Console.WriteLine("\n");
        }
        private static void printFormPanel(WSFormPanelBean panel)
        {
            Console.WriteLine("************************************************");
            Console.WriteLine("PANEL: Rows=" + panel.rows + "\tCols=" + panel.cols);
            WSFormFieldBean[] fields = panel.fields;
            for (int i = 0; i < fields.Length; i++)
            {
                WSFormFieldBean fieldBean = fields[i];
                Console.WriteLine("FORM_FIELD: FieldID=" + fieldBean.fieldID + " POS(" + fieldBean.rowIndex + "," + fieldBean.colIndex + ") SPAN(" + fieldBean.rowSpan + "," + fieldBean.colSpan + ")" +
                        " Label_ALIGN(" + getHorizontalAlign(fieldBean.labelHAlign) + "," + getVerticalAlign(fieldBean.labelVAlign) + ")" +
                        " Value_ALIGN(" + getHorizontalAlign(fieldBean.valueHAlign) + "," + getVerticalAlign(fieldBean.valueVAlign) + ")");
            }
            Console.WriteLine("************************************************\n");
        }
        public static String getVerticalAlign(String valignStr)
        {
            int valign = TCLFacade.TOP;
            int.TryParse(valignStr, out valign);
            return getVerticalAlign(valign);
        }
        public static String getVerticalAlign(int valign)
        {
            switch (valign)
            {
                case TCLFacade.MIDDLE:
                    {
                        return "middle";
                    }
                case TCLFacade.TOP:
                    {
                        return "top";
                    }
                case TCLFacade.BOTTOM:
                    {
                        return "bottom";
                    }
            }
            return "top";
        }
        public static String getHorizontalAlign(String halignStr)
        {
            int halign = TCLFacade.LEFT;
            int.TryParse(halignStr, out halign);
            return getHorizontalAlign(halign);
        }
        public static String getHorizontalAlign(int halign)
        {
            switch (halign)
            {
                case TCLFacade.CENTER:
                    {
                        return "center";
                    }
                case TCLFacade.LEFT:
                    {
                        return "left";
                    }
                case TCLFacade.RIGHT:
                    {
                        return "right";
                    }
            }
            return "left";
        }


        public static void runFormSample(String[] args)
        {
            FormSample formSample = new FormSample();
            formSample.login();

            formSample.getEditForm();
        }

    }
}
