/*******************************************************************************
 * Copyright (C) 2015 Steinbeis GmbH & Co. KG.
 * All rights reserved. 
 *
 * This program and the accompanying materials are made 
 * available under the terms of the Track+ License v2.0
 * which is available at http://www.trackplus.com/tpl-v20.html
 *
 * $Id:$
 *******************************************************************************/


using System;
using System.Collections.Generic;
using System.Text;

using com.trackplus.webservice.client.trackplusWS;
using com.trackplus.track.ws;
using com.trackplus.track.ws.tcl;
using com.trackplus.track.ws.util;
using log4net;
using log4net.Config;

namespace com.trackplus.track.ws{

public class GroupSample : LoginSample {

    private static readonly ILog log = LogManager.GetLogger(typeof(GroupSample));

	public GroupSample() : base() {	
	}

	public GroupSample(TCLFacade connector) : base(connector) {	
	}

	public WSGroupBean[] getAllGroups() {
		try {
			WSGroupBean[] allGroups = connector.getAllGroups();
			return allGroups;
		} catch (TCLFacadeException e) {
			log.Error(e);
			return null;
		} catch (Exception e) {
			log.Error(e);
			return null;
		}
	}

	public String addGroup(String groupName) { // throws RemoteException, TCLFacadeException {
		WSGroupEditBean newGroup = new WSGroupEditBean();
		newGroup.name = groupName;
		newGroup.inheritAuthor = true;
		newGroup.inheritManager = false;
		newGroup.inheritResponsible = true;
		String groupID = connector.saveGroup(newGroup);
		return groupID;
	}

	private void printGroups(WSGroupBean[] allGroups) {
		if (allGroups != null) {
			WSGroupBean group;
			for (int i = 0; i < allGroups.Length; i++) {
				group = allGroups[i];
				Console.WriteLine(i + " {name=" + group.name + " ;ID=" + group.objectID + "}");
			}
		} else {
			Console.WriteLine("No groups");
		}
	}

	public WSGroupEditBean getGroupEdit(String groupID) {
		try {
			WSGroupEditBean groupEditBean = connector.getEditGroup(groupID);
			return groupEditBean;
		} catch (TCLFacadeException e) {
			log.Error(e);
		} catch (Exception e) {
			log.Error(e);
		}
		return null;
	}

	public void deleteGroup(String groupID) {
		try {
			connector.deleteGroup(groupID);
		} catch (TCLFacadeException e) {
			log.Error(e);
		} catch (Exception e) {
			log.Error(e);
		}
	}

	public void addPersonsToGroup(String groupID, String[] personIDs) {
		try {
			connector.addPersonsToGroup(groupID, personIDs);
		} catch (TCLFacadeException e) {
			log.Error(e);
		} catch (Exception e) {
			log.Error(e);
		}
	}

	public void removePersonsFromGroup(String groupID, String[] personIDs) {
		try {
			connector.removePersonsFromGroup(groupID, personIDs);
		} catch (TCLFacadeException e) {
			log.Error(e);
		} catch (Exception e) {
			log.Error(e);
		}
	}

	public WSPersonBean[] getPersonsForGroup(String groupID) {
		try {
			WSPersonBean[] persons = connector.getPersonsForGroup(groupID);
			return persons;
		} catch (TCLFacadeException e) {
			log.Error(e);
		} catch (Exception e) {
			log.Error(e);
		}
        return null;
	}

	private void printEditGroup(WSGroupEditBean groupEditBean) {
		Console.WriteLine("WSGroupEditBean:[");
		Console.WriteLine("objectID=" + groupEditBean.objectID);
		Console.WriteLine("name=" + groupEditBean.name);
		Console.WriteLine("originator=" + groupEditBean.inheritAuthor);
		Console.WriteLine("manager=" + groupEditBean.inheritManager);
		Console.WriteLine("responsible=" + groupEditBean.inheritResponsible);
		Console.WriteLine("]");
	}

	public static void runGroupSample(String[] args) {
		GroupSample groupSample = new GroupSample();
		// login
		groupSample.login(user, passwd);
		WSGroupBean[] allGroups = groupSample.getAllGroups();
		Console.WriteLine("Initial groups:");
		groupSample.printGroups(allGroups);
		Console.WriteLine("\n");
		try {
			String groupID = groupSample.addGroup("group1");
			Console.WriteLine("Group saved with id:" + groupID + "\n");

			WSGroupEditBean groupEditBean = groupSample.getGroupEdit(groupID);
			Console.WriteLine("new group added:");
			groupSample.printEditGroup(groupEditBean);
			Console.WriteLine("\n");

			allGroups = groupSample.getAllGroups();
			Console.WriteLine("After add:");
			groupSample.printGroups(allGroups);
			Console.WriteLine("\n");

			PersonSample personSample = new PersonSample(groupSample.connector);
			String personID1 = personSample.createUser(1);
			String personID2 = personSample.createUser(2);
			groupSample.addPersonsToGroup(groupID, new String[] { personID1, personID2 });

			WSPersonBean[] personInGroups = groupSample.getPersonsForGroup(groupID);
			Console.WriteLine("Person in group:" + groupID);
			personSample.printPersons(personInGroups);

			groupSample.removePersonsFromGroup(groupID, new String[] { personID1 });
			personInGroups = groupSample.getPersonsForGroup(groupID);
			Console.WriteLine("Person in group:" + groupID + " afer remove first person");
			personSample.printPersons(personInGroups);

			groupSample.removePersonsFromGroup(groupID, new String[] { personID2 });
			personSample.deletePerson(personID1);
			personSample.deletePerson(personID2);

			groupSample.deleteGroup(groupID);
			Console.WriteLine("After delete:");
			allGroups = groupSample.getAllGroups();
			groupSample.printGroups(allGroups);

		} catch (TCLFacadeException e) {
			log.Error(e);
		} catch (Exception e) {
			log.Error(e);
		}
	}

  }

}
