
using System;
using System.Collections.Generic;
using System.Text;

using com.trackplus.webservice.client.trackplusWS;
using com.trackplus.track.ws;
using com.trackplus.track.ws.tcl;
using com.trackplus.track.ws.util;
using log4net;
using log4net.Config;

namespace com.trackplus.track.ws
{
    /**
     */
    public class ItemLinkSample : ItemSample
    {

        private static readonly ILog log = LogManager.GetLogger(typeof(ItemLinkSample));

        /**
         * The constructor.
         */
        public ItemLinkSample()
            : base()
        {

        }

        /**
         * Constructor with existing connection.
         * @param connector
         */
        public ItemLinkSample(TCLFacade connector)
            : base(connector)
        {

        }

        public WSItemLinkBean[] getItemLinks(String itemID)
        {
            try
            {
                WSItemLinkBean[] links = connector.getItemLinks(itemID);
                return links;
            }
            catch (TCLFacadeException e)
            {
                log.Error(e);
                return null;
            }
            catch (Exception e)
            {
                log.Error(e);
                return null;
            }
        }

        private void printItemLinks(WSItemLinkBean[] links)
        {
            if (links != null)
            {
                for (int i = 0; i < links.Length; i++)
                {
                    Console.WriteLine("Item link:" + i);
                    WSItemLinkBean link = links[i];
                    printItemLink(link);
                    Console.WriteLine("\n\n");
                }
            }
        }

        private void printItemLink(WSItemLinkBean itemLinkBean)
        {
            Console.WriteLine("objectID=" + itemLinkBean.objectID);
            Console.WriteLine("linkType=" + itemLinkBean.linkType);
            Console.WriteLine("linkTypeName=" + itemLinkBean.linkTypeName);
            Console.WriteLine("linkedWorkItemID=" + itemLinkBean.linkedWorkItemID);
            Console.WriteLine("linkedWorkItemTitle=" + itemLinkBean.linkedWorkItemTitle);
            Console.WriteLine("editable=" + itemLinkBean.editable);
            Console.WriteLine("sortOrder=" + itemLinkBean.sortOrder);
            Console.WriteLine("description=" + itemLinkBean.description);
            Console.WriteLine("externalLink=" + itemLinkBean.externalLink);
            Console.WriteLine("changedBy=" + itemLinkBean.changedBy);
            Console.WriteLine("lastEdit=" + itemLinkBean.lastEdit);

            WSKeyValueBean[] parameters = itemLinkBean.parameters;
            if (parameters == null || parameters.Length == 0)
            {
                Console.WriteLine("No params");
            }
            else
            {
                Console.WriteLine("parameters=" + parameters.Length);
            }

        }

        public WSOptionBean[] getLinkTypes()
        {
            try
            {
                WSOptionBean[] linkTypes = connector.getLinkTypes();
                return linkTypes;
            }
            catch (TCLFacadeException e)
            {
                log.Error(e);
                return null;
            }
            catch (Exception e)
            {
                log.Error(e);
                return null;
            }
        }

        public void printOptions(WSOptionBean[] options)
        {
            if (options != null)
            {
                for (int i = 0; i < options.Length; i++)
                {
                    Console.WriteLine("ID=" + options[i].objectID + " Label=" + options[i].label);
                }
            }
        }

        public WSParameter[] getLinkTypeParams(String linkTypeID)
        {
            try
            {
                WSParameter[] parameters = connector.getLinkTpeParameters(linkTypeID);
                return parameters;
            }
            catch (TCLFacadeException e)
            {
                log.Error(e);
                return null;
            }
            catch (Exception e)
            {
                log.Error(e);
                return null;
            }
        }
        public void printParameters(WSParameter[] parameters)
        {
            if (parameters == null || parameters.Length == 0)
            {
                Console.WriteLine("NO params");
            }
            else
            {
                WSParameter parameter;
                Console.WriteLine("params size:" + parameters.Length);
                for (int i = 0; i < parameters.Length; i++)
                {
                    parameter = parameters[i];
                    if (parameter != null)
                    {
                        Console.WriteLine("parameter=" + parameter);
                        Console.WriteLine("name=" + parameter.name);
                        Console.WriteLine("type=" + parameter.type);
                        Console.WriteLine("required=" + parameter.required);

                        WSOptionBean[] options = parameter.options;
                        Console.WriteLine("options:");
                        printOptions(options);
                        Console.WriteLine("\n\n");
                    }
                }
            }
        }
        public String saveItemLink(String itemID, WSItemLinkBean itemLinkBean)
        {
            try
            {
                return connector.saveItemLink(itemID, itemLinkBean);
            }
            catch (TCLFacadeException e)
            {
                log.Error(e);
                return null;
            }
            catch (Exception e)
            {
                log.Error(e);
                return null;
            }
        }
        public WSItemLinkBean getItemLink(String itemID, String itemLinkID)
        {
            try
            {
                return connector.getItemLink(itemID, itemLinkID);
            }
            catch (TCLFacadeException e)
            {
                log.Error(e);
                return null;
            }
            catch (Exception e)
            {
                log.Error(e);
                return null;
            }
        }

        public void deleteItemLinks(String itemID, String[] itemLinkArray)
        {
            try
            {
                connector.deleteItemLinks(itemID, itemLinkArray);
            }
            catch (TCLFacadeException e)
            {
                log.Error(e);
                return;
            }
            catch (Exception e)
            {
                log.Error(e);
                return;
            }
        }


        public static void runItemLinkSample(String[] args)
        {
            ItemLinkSample itemLinkSample = new ItemLinkSample();
            itemLinkSample.login(user, passwd);

            String itemID=itemLinkSample.createItem("My dummy item");
            // String itemID = "3";

            if (itemID != null)
            {
                WSOptionBean[] linkTypes = itemLinkSample.getLinkTypes();
                Console.WriteLine("LinkTypes:");
                itemLinkSample.printOptions(linkTypes);
                Console.WriteLine("\n\n\n*********************\n\n\n\n");
                String linkTypeID = null;
                if (linkTypes != null && linkTypes.Length > 0)
                {
                    WSOptionBean linkType = linkTypes[0];
                    linkTypeID = linkType.objectID;
                    WSParameter[] parameters = itemLinkSample.getLinkTypeParams(linkTypeID);
                    Console.WriteLine("Parameters for link type:" + linkTypeID);
                    itemLinkSample.printParameters(parameters);
                    Console.WriteLine("\n\n\n*********************\n\n\n\n");
                }

                //create item Link
                String newItemLinkID = null;
                if (linkTypeID != null)
                {
                    WSItemLinkBean itemLinkBean = new WSItemLinkBean();
                    String linkedItemID=itemLinkSample.createItem("LinkedItem");
                    // String linkedItemID = "65";
                    if (linkedItemID != null)
                    {
                        itemLinkBean.linkType = linkTypeID;
                        itemLinkBean.linkedWorkItemID = linkedItemID;
                        itemLinkBean.description = "some item link form webservice";

                        newItemLinkID = itemLinkSample.saveItemLink(itemID, itemLinkBean);
                        Console.WriteLine("ItemLink saved with id:" + newItemLinkID);
                        itemLinkBean = itemLinkSample.getItemLink(itemID, newItemLinkID);
                        Console.WriteLine(" ITEM LINK bean:");
                        itemLinkSample.printItemLink(itemLinkBean);
                    }
                }

                //get item Links
                WSItemLinkBean[] links = itemLinkSample.getItemLinks(itemID);
                int size = links == null ? 0 : links.Length;
                Console.WriteLine("All item links :size=" + size);
                itemLinkSample.printItemLinks(links);


                //delete itemLinks
                String[] deletedItemLinks = new String[] { newItemLinkID };
                itemLinkSample.deleteItemLinks(itemID, deletedItemLinks);

                links = itemLinkSample.getItemLinks(itemID);
                size = links == null ? 0 : links.Length;
                Console.WriteLine("AGAIN all item links after delete: size=" + size);
                itemLinkSample.printItemLinks(links);

            }
        }

    }
}
