/*******************************************************************************
 * Copyright (C) 2015 Steinbeis GmbH & Co. KG.
 * All rights reserved. 
 *
 * This program and the accompanying materials are made 
 * available under the terms of the Track+ License v2.0
 * which is available at http://www.trackplus.com/tpl-v20.html
 *
 * $Id:$
 *******************************************************************************/

using System;
using System.Collections.Generic;
using System.Text;

using com.trackplus.webservice.client.trackplusWS;
using com.trackplus.track.ws;
using com.trackplus.track.ws.tcl;
using com.trackplus.track.ws.util;
using log4net;
using log4net.Config;

namespace com.trackplus.track.ws
{

    /**
     * This class provides handling for custom selection lists (tags, labels). You
     * can add, remove, and modify lists.
     * 
     */
    public class ListSample : LoginSample
    {

        private static readonly ILog log = LogManager.GetLogger(typeof(ListSample));

        /**
         * Constructor
         */
        public ListSample()
            : base()
        {

        }

        public ListSample(TCLFacade connector)
            : base(connector)
        {

        }

        /**
         * Get all lists available to you.
         * 
         */
        public WSListBean[] getGlobalLists()
        {
            try
            {
                WSListBean[] lists = connector.getGlobalLists();
                if (lists != null)
                {
                    WSListBean list;
                    for (int i = 0; i < lists.Length; i++)
                    {
                        list = lists[i];
                        Console.WriteLine(i + " {name=" + list.name + "; objectID=" + list.objectID + "}");
                    }
                }
                else
                {
                    Console.WriteLine("No global list");
                }
                return lists;
            }
            catch (TCLFacadeException e)
            {
                log.Error(e);
                return null;
            }
            catch (Exception e)
            {
                log.Error(e);
                return null;
            }
        }

        /**
         * Add a selection list. Later you can ad options to this list.
         * 
         */
        public String addList()
        {
            WSListBean list = new WSListBean();
            list.name = "myListFromWebservice";
            list.description = "list created from web-service";
            try
            {
                return connector.saveList(list);

            }
            catch (TCLFacadeException e)
            {
                log.Error(e);
                return null;
            }
            catch (Exception e)
            {
                log.Error(e);
                return null;
            }
        }

        /**
         * Retrieve all options for a given list.
         * 
         * @param listID
         */
        public WSListOptionBean[] getOptionsByList(String listID)
        {
            try
            {
                WSListOptionBean[] options = connector.getListOptions(listID);
                return options;
            }
            catch (TCLFacadeException e)
            {
                log.Error(e);
                return null;
            }
            catch (Exception e)
            {
                log.Error(e);
                return null;
            }
        }

        /**
         * Delete an option by its object id.
         * 
         * @param optionID
         */
        public void deleteOption(String optionID)
        {
            try
            {
                connector.deletListOption(optionID);
            }
            catch (TCLFacadeException e)
            {
                log.Error(e);
                return;
            }
            catch (Exception e)
            {
                log.Error(e);
                return;
            }
        }

        private void printOptions(WSListOptionBean[] options)
        {
            if (options != null)
            {
                WSListOptionBean option;
                for (int i = 0; i < options.Length; i++)
                {
                    option = options[i];
                    Console.WriteLine(i + " {name=" + option.label + "; objectID=" + option.objectID + "}");
                }
            }
            else
            {
                Console.WriteLine("null options");
            }
        }

        /**
         * Add an option to a given list.
         * 
         * @param listID
         *            the object id of the list this option should be added to
         * @param label
         *            the label for this option
         */
        public String addListOption(String listID, String label)
        {
            WSListOptionBean option = new WSListOptionBean();
            option.listID = listID;
            option.label = label;
            try
            {
                return connector.saveListOption(option);
            }
            catch (TCLFacadeException e)
            {
                log.Error(e);
                return null;
            }
            catch (Exception e)
            {
                log.Error(e);
                return null;
            }
        }

        public static void runListSample(String[] args)
        {
            String url;
            if (args.Length >= 2)
            {
                user = (args[0]);
                passwd = args[1];
            }
            if (args.Length > 2)
            {
                url = args[3];
            }
            ListSample listSample = new ListSample();
            // login
            listSample.login(user, passwd);

            String listID = listSample.addList();
            String option1ID = null;

            if (listID != null)
            {
                option1ID = listSample.addListOption(listID, "option1");
                listSample.addListOption(listID, "option2");
            }
            WSListBean[] globalLists = listSample.getGlobalLists();
            if (globalLists != null && globalLists.Length > 0)
            {
                Console.WriteLine("\n\n");
                WSListBean listBean;
                for (int i = 0; i < globalLists.Length; i++)
                {
                    listBean = globalLists[i];
                    listID = listBean.objectID;
                    WSListOptionBean[] options = listSample.getOptionsByList(listID);
                    Console.WriteLine("List " + listID + ": " + listBean.name);
                    Console.WriteLine("Options:");
                    listSample.printOptions(options);
                    Console.WriteLine("\n");
                }

                if (option1ID != null)
                {
                    listSample.deleteOption(option1ID);
                    WSListOptionBean[] options = listSample.getOptionsByList(listID);
                    Console.WriteLine("Options after delete:");
                    listSample.printOptions(options);
                }
            }
        }
    }
}

