/*
 * This file is part of the Track+ application, a
 * tool for project and change management.
 *
 * Copyright (C) 2008 Trackplus
 *
 * Use and distribution of this software is governed by the
 * Track+ license conditions.
 *
 * $Id$
 */


using System;
using System.Collections.Generic;
using System.Text;

using com.trackplus.webservice.client.trackplusWS;
using com.trackplus.track.ws;
using com.trackplus.track.ws.tcl;
using com.trackplus.track.ws.util;
using log4net;
using log4net.Config;


namespace com.trackplus.track.ws
{

    public class PersonSample : LoginSample
    {

        private static readonly ILog log = LogManager.GetLogger(typeof(PersonSample));

        public PersonSample()
            : base()
        {

        }

        public PersonSample(TCLFacade connector)
            : base(connector)
        {

        }

        /**
         * Return all users in the database in an array. This is for view only, not for editing
         * a user entry.
         * @return all users in the datbase
         */
        public WSPersonBean[] getAllUsers()
        {
            try
            {
                WSPersonBean[] allPersons = connector.getAllPersons();
                printPersons(allPersons);
                return allPersons;
            }
            catch (TCLFacadeException e)
            {
                log.Error(e);
                return null;
            }
            catch (Exception e)
            {
                log.Error(e);
                return null;
            }
        }

        public void printPersons(WSPersonBean[] allPersons)
        {
            if (allPersons != null)
            {
                WSPersonBean person;
                for (int i = 0; i < allPersons.Length; i++)
                {
                    person = allPersons[i];
                    Console.WriteLine(i + " {Name=" + person.fullName + " ;  personID=" + person.personID
                                        + "; loginName=" + person.loginName + "}");
                }
            }
        }

        /**
         * Get a bean of a user based on his object id. The result is suitable for
         * changing the users properties and saving back into the database.
         * @param personID the object id of the user
         * @return a bean with all user properties
         */
        public WSPersonEditBean getPersonEdit(String personID)
        {
            try
            {
                WSPersonEditBean personEditBean = connector.getEditPerson(personID);
                if (personEditBean != null)
                {
                    printEditPerson(personEditBean);
                }
                return personEditBean;
            }
            catch (TCLFacadeException e)
            {
                log.Error(e);
                return null;
            }
            catch (Exception e)
            {
                log.Error(e);
                return null;
            }
        }

        /**
         * Get a bean of a user based on his login name. The result is suitable for
         * changing the users properties and saving back into the database.
         * @param loginName the login name of the user
         * @return a bean with all user properties
         */
        public WSPersonEditBean getPersonEditByLoginName(String loginName)
        {
            Console.WriteLine("/nLoading person by loginName:" + loginName);
            try
            {
                WSPersonEditBean personEditBean = connector.getEditPersonByLoginName(loginName);
                if (personEditBean != null)
                {
                    printEditPerson(personEditBean);
                }
                return personEditBean;
            }
            catch (TCLFacadeException e)
            {
                log.Error(e);
                return null;
            }
            catch (Exception e)
            {
                log.Error(e);
                return null;
            }
        }

        /**
         * Delete a user from the database based on its object id.
         * @param personID the object id of the user to be deleted.
         */
        public void deletePerson(String personID)
        {
            try
            {
                connector.deletePerson(personID);
            }
            catch (TCLFacadeException e)
            {
                log.Error(e);
            }
            catch (Exception e)
            {
                log.Error(e);
            }
        }

        /*
         * Just a helper method showing the user bean properties available.
         */
        private void printEditPersonContext(WSPersonEditWithContext personEditContext)
        {
            Console.WriteLine("locales=" + personEditContext.locales);
            Console.WriteLine("timeZones=" + personEditContext.timeZones);
            Console.WriteLine("remindPriorityLevels=" + personEditContext.remindPriorityLevels);
            Console.WriteLine("remindSeverityLevels=" + personEditContext.remindSeverityLevels);
            Console.WriteLine("remindMeOnDaysList=" + personEditContext.remindMeOnDaysList);
            Console.WriteLine("departmentTree=" + personEditContext.departmentTree);
            Console.WriteLine("designPaths=" + personEditContext.designPaths);
            Console.WriteLine("substitutePersonsList=" + personEditContext.substitutePersonsList);
        }

        private void printEditPerson(WSPersonEditBean personEditBean)
        {
            Console.WriteLine("\nWSPersonEditBean:[");
            Console.WriteLine("objectID=" + personEditBean.objectID);
            Console.WriteLine("userName=" + personEditBean.userName);
            Console.WriteLine("firstName=" + personEditBean.firstName);
            Console.WriteLine("lastName=" + personEditBean.lastName);
            Console.WriteLine("userEmail=" + personEditBean.userEmail);
            Console.WriteLine("ldapOn=" + personEditBean.ldapOn);
            Console.WriteLine("forceLdap=" + personEditBean.forceLdap);
            Console.WriteLine("ldapUser=" + personEditBean.ldapUser);
            Console.WriteLine("locale=" + personEditBean.locale);


            Console.WriteLine("adminOrGuest=" + personEditBean.adminOrGuest);
            Console.WriteLine("domainPat=" + personEditBean.domainPat);
            Console.WriteLine("userTz=" + personEditBean.userTz);


            Console.WriteLine("noEmail=" + personEditBean.noEmail);
            Console.WriteLine("emailLead=" + personEditBean.emailLead);
            Console.WriteLine("remindMeAsOriginator=" + personEditBean.remindMeAsOriginator);
            Console.WriteLine("remindMeAsManager=" + personEditBean.remindMeAsManager);
            Console.WriteLine("remindMeAsResponsible=" + personEditBean.remindMeAsResponsible);

            Console.WriteLine("remindPriorityLevel=" + personEditBean.remindPriorityLevel);
            Console.WriteLine("remindSeverityLevel=" + personEditBean.remindSeverityLevel);
            Console.WriteLine("prefEmailType=" + personEditBean.prefEmailType);
            Console.WriteLine("remindMeOnDays=" + personEditBean.remindMeOnDays);

            Console.WriteLine("phone=" + personEditBean.phone);

            Console.WriteLine("department=" + personEditBean.department);
            Console.WriteLine("workingHours=" + personEditBean.workingHours);
            Console.WriteLine("employeeId=" + personEditBean.employeeId);
            Console.WriteLine("lastLogin=" + personEditBean.lastLogin);
            Console.WriteLine("csvSeparator=" + personEditBean.csvSeparator);
            Console.WriteLine("csvEncoding=" + personEditBean.csvEncoding);
            Console.WriteLine("saveAttachments=" + personEditBean.saveAttachments);
            Console.WriteLine("sessionTimeout=" + personEditBean.sessionTimeout);

            Console.WriteLine("designPath=" + personEditBean.designPath);
            Console.WriteLine("activateInlineEdit=" + personEditBean.activateInlineEdit);
            Console.WriteLine("activateLayout=" + personEditBean.activateLayout);
            Console.WriteLine("homePage=" + personEditBean.homePage);

            Console.WriteLine("substituteID=" + personEditBean.substituteID);

            Console.WriteLine("]\n");

        }
        public String createUser(int index)
        {
            WSPersonEditBean personEditBean = new WSPersonEditBean();
            personEditBean.firstName = "firstName" + index;
            personEditBean.lastName = "lastName" + index;
            personEditBean.userEmail = "user" + index + "@trackplus.com";
            personEditBean.userName = "user" + index;
            personEditBean.passwd = "user" + index;
            personEditBean.passwd2 = "user" + index;
            String personID = null;
            try
            {
                personID = connector.savePerson(personEditBean);
            }
            catch (TCLFacadeException e)
            {
                log.Error(e);
            }
            catch (Exception e)
            {
                log.Error(e);
            }
            return personID;
        }


        private void replaceAndDeletePerson()
        {
            ItemSample itemSample = new ItemSample(connector);
            String personID1 = createUser(1);
            if (personID1 != null)
            {
                Console.WriteLine("User created:" + personID1);
                //create an item and  set user1 as responsible
                String itemID = itemSample.createItem();
                if (itemID != null)
                {
                    Console.WriteLine("Item created:" + itemID);
                    WSItemBean itemBean = null;
                    try
                    {
                        itemBean = connector.getItem(itemID);
                    }
                    catch (TCLFacadeException e)
                    {
                        log.Error(e);
                    }
                    catch (Exception e)
                    {
                        log.Error(e);
                    }
                    if (itemBean != null)
                    {
                        itemBean.responsibleID = personID1;
                        try
                        {
                            connector.saveItem(itemBean);
                        }
                        catch (TCLFacadeException e)
                        {
                            log.Error(e);
                        }
                        catch (Exception e)
                        {
                            log.Error(e);
                        }
                    }
                }

                //try to delete user
                try
                {
                    connector.deletePerson(personID1);
                    Console.WriteLine("Person was simple deleted!");
                }
                catch (TCLFacadeException e)
                {
                    if (e.getErrorCode() == TCLFacadeException.DELETE_ERROR_NEED_REPLACE)
                    {
                        Console.WriteLine("The user can't be deleted, there is need for replacement.");
                        String personID2 = createUser(2);
                        Console.WriteLine("Create new user:" + personID2);
                        //replace and delete person
                        try
                        {
                            connector.replaceAndDeletePerson(personID1, personID2);
                            Console.WriteLine("Person succesfully replaced");
                        }
                        catch (TCLFacadeException ei)
                        {
                            log.Error(ei);
                        }
                        catch (Exception ex)
                        {
                            log.Error(ex);
                        }

                    }
                    else
                    {
                        log.Error(e);
                    }
                }
                catch (Exception e)
                {
                    log.Error(e);
                }

            }
        }

        /**
         * Demonstrator for how to add, view, modify, and delete users.
         * @param args
         */
        public static void runPersonSample(String[] args)
        {
            PersonSample personSample = new PersonSample();
            //login
            personSample.login(user, passwd);

            WSPersonBean[] allPersons = personSample.getAllUsers();
            personSample.printPersons(allPersons);
            if (allPersons != null && allPersons.Length > 0)
            {
                String personID = allPersons[1].personID;
                String loginName = allPersons[1].loginName;

                personSample.getPersonEditByLoginName(loginName);

            }
            personSample.replaceAndDeletePerson();

        }
    }
}
