/*******************************************************************************
 * Copyright (C) 2015 Steinbeis GmbH & Co. KG.
 * All rights reserved. 
 *
 * This program and the accompanying materials are made 
 * available under the terms of the Track+ License v2.0
 * which is available at http://www.trackplus.com/tpl-v20.html
 *
 * $Id:$
 *******************************************************************************/

using System;
using System.Collections.Generic;
using System.Text;

using com.trackplus.webservice.client.trackplusWS;
using com.trackplus.track.ws;
using com.trackplus.track.ws.tcl;
using com.trackplus.track.ws.util;
using log4net;
using log4net.Config;

namespace com.trackplus.track.ws
{

    public class ProjectSample : LoginSample

    {
        private static readonly ILog log = LogManager.GetLogger(typeof(ProjectSample));

        public ProjectSample()
            : base()
        {

        }

        public ProjectSample(TCLFacade connector)
            : base(connector)
        {

        }

        public WSLabelValueBean[] getProjects()
        {
            WSLabelValueBean[] projects = null;
            try
            {
                projects = connector.getProjects();
            }
            catch (TCLFacadeException e)
            {
                handleTCLFacadeException(e);
            }
            catch (Exception e)
            {
                log.Error(e);
            }

            return projects;
        }

        public WSLabelValueBean[] getAllProjects()
        {
            WSLabelValueBean[] projects = null;
            try
            {
                projects = connector.getAllProjects();
            }
            catch (TCLFacadeException e)
            {
                handleTCLFacadeException(e);
            }
            catch (Exception e)
            {
                log.Error(e);
            }

            return projects;
        }

        public WSTreeNode[] getProjectsAsTree(){
		Console.WriteLine("\n->ProjectSample.getProjectsAsTree()");
		WSTreeNode[] projects=null;
		try {
			projects=connector.getProjectsAsTree();
			if(projects==null||projects.Length==0){
				Console.WriteLine("No projects found on the server. Maybe the user does not have right in any project");
			}else{
				Console.WriteLine("Successfully obtained "+projects.Length+ " projects:");
				for (int i = 0; i < projects.Length; i++) {
					BaseOptionSample.printNode(projects[i],"");
				}
				Console.WriteLine();
			}
        }
        catch (TCLFacadeException e)
        {
            log.Error(e);
        }
        catch (Exception e)
        {
            log.Error(e);
        }
		Console.WriteLine("<-ProjectSample.getProjectsAsTree()\n");
		return projects; 
	}


        public String addProject()
        {
            return addProject("");
        }

        public String addProject(String suffix)
        {
            try
            {
                WSProjectEditWithContext projectEditContextBean = connector.getNewProjectWithContext();
                WSProjectEditBean projectEditBean = new WSProjectEditBean();

                WSLabelValueBean[] projectTypes = projectEditContextBean.projectTypeList;
                WSLabelValueBean[] projectStateList = projectEditContextBean.projectStateList;

                String projectName = "my project" + suffix;
                projectEditBean.name = projectName;
                String projectTypeID = projectTypes[0].value;
                projectEditBean.projectType = projectTypeID;
                String projectStateID = projectStateList[0].value;
                projectEditBean.projectState = projectStateID;

                String projectID = connector.saveProject(projectEditBean);
                return projectID;
            }
            catch (TCLFacadeException e)
            {
                log.Error(e);
            }
            catch (Exception e)
            {
                log.Error(e);
            }
            return null;
        }

        public WSProjectEditBean getProjectEdit(String projectID)
        {
            try
            {
                WSProjectEditBean projectEditBean = connector.getEditProject(projectID);
                if (projectEditBean != null)
                {
                    printEditProject(projectEditBean);
                }
                return projectEditBean;
            }
            catch (TCLFacadeException e)
            {
                log.Error(e);
            }
            catch (Exception e)
            {
                log.Error(e);
            }

            return null;
        }


        public void deleteProject(String projectID)
        {
            try
            {
                connector.deleteProject(projectID);
            }
            catch (TCLFacadeException e)
            {
                log.Error(e);
            }
            catch (Exception e)
            {
                log.Error(e);
            }
        }

        private void printAllProject(WSLabelValueBean[] allProjects) {
		if(allProjects!=null){
			WSLabelValueBean bean;
			for(int i=0;i<allProjects.Length;i++){
				bean=allProjects[i];
				Console.WriteLine(i+" {name="+bean.label+" ;ID="+bean.value+"}");
			}
		}
	}

        private void printEditProject(WSProjectEditBean projectEditBean) {
		Console.WriteLine("\nWSProjectEditBean:[");
		Console.WriteLine("objectID="+projectEditBean.objectID);
				
	
		
		Console.WriteLine("]\n");
		
	}

        public static void runProjectSample(String[] args)
        {
            ProjectSample projectSample = new ProjectSample();
            //login
            projectSample.login(user, passwd);
            WSTreeNode[] projectsTree = projectSample.getProjectsAsTree();
            if (projectsTree != null)
            {
                String projectID = projectsTree[0].id;
                try
                {
                    WSProjectEditBean projectEditBean = projectSample.getProjectEdit(projectID);
                }
                catch (Exception e)
                {
                    log.Error(e);
                }
            }

            projectSample.addProject();
            WSLabelValueBean[] allProjects = projectSample.getProjects();
            projectSample.printAllProject(allProjects);


        }

    }
}
