/*******************************************************************************
 * Copyright (C) 2015 Steinbeis GmbH & Co. KG.
 * All rights reserved. 
 *
 * This program and the accompanying materials are made 
 * available under the terms of the Track+ License v2.0
 * which is available at http://www.trackplus.com/tpl-v20.html
 *
 * $Id:$
 *******************************************************************************/


using System;
using System.Collections.Generic;
using System.Text;

using com.trackplus.webservice.client.trackplusWS;
using com.trackplus.track.ws;
using com.trackplus.track.ws.tcl;
using com.trackplus.track.ws.util;
using log4net;
using log4net.Config;

namespace com.trackplus.track.ws
{

    public class RoleSample : LoginSample
    {

        private static readonly ILog log = LogManager.GetLogger(typeof(RoleSample));

        /**
         * The constructor
         */
        public RoleSample()
            : base()
        {
        }

        public RoleSample(TCLFacade connector)
            : base(connector)
        {

        }

        /**
         * Get an array with all roles with label as role name and value as role id.
         * 
         */
        public WSLabelValueBean[] getAllRoles()
        {
            try
            {
                WSLabelValueBean[] allRoles = connector.getAllRoles();
                return allRoles;
            }
            catch (TCLFacadeException e)
            {
                log.Error(e);
                return null;
            }
            catch (Exception e)
            {
                log.Error(e);
                return null;
            }
        }

        /**
         * Get a array with all role assignments for all users and groups for a
         * given project or workspace.
         * 
         * @param projectID
         *            the object id for the workspace to get the role assignments
         *            for
         */
        public WSRoleAssignmentBean[] getAssignmentRolesByProject(String projectID)
        {
            try
            {
                WSRoleAssignmentBean[] roles = connector.getAssignmentRolesByProject(projectID);
                return roles;
            }
            catch (TCLFacadeException e)
            {
                log.Error(e);
                return null;
            }
            catch (Exception e)
            {
                log.Error(e);
                return null;
            }
        }

        /**
         * Add a role assignment.
         * 
         * @param roleAssignmentEditBean
         */
        public void addRoleAssignment(WSRoleAssignmentEditBean roleAssignmentEditBean)
        {
            try
            {
                connector.addRoleAssignment(roleAssignmentEditBean);
            }
            catch (TCLFacadeException e)
            {
                log.Error(e);
                return;
            }
            catch (Exception e)
            {
                log.Error(e);
                return;
            }
        }

        /**
         * Remove a role assignment.
         * 
         * @param roleAssignmentEditBean
         */
        public void removeRoleAssignment(WSRoleAssignmentEditBean roleAssignmentEditBean)
        {
            try
            {
                connector.removeRoleAssignment(roleAssignmentEditBean);
            }
            catch (TCLFacadeException e)
            {
                log.Error(e);
                return;
            }
            catch (Exception e)
            {
                log.Error(e);
                return;
            }
        }

        private Dictionary<String, String> labelValueBeansAsMap(WSLabelValueBean[] beans)
        {
            Dictionary<String, String> map = new Dictionary<String, String>();
            if (beans != null)
            {
                WSLabelValueBean bean;
                for (int i = 0; i < beans.Length; i++)
                {
                    bean = beans[i];
                    map[bean.value] = bean.label;
                }
            }
            return map;
        }

        private Dictionary<String, String> personBeansAsMap(WSPersonBean[] beans)
        {
            Dictionary<String, String> map = new Dictionary<String, String>();
            if (beans != null)
            {
                WSPersonBean bean;
                for (int i = 0; i < beans.Length; i++)
                {
                    bean = beans[i];
                    map[bean.personID] = bean.fullName;
                }
            }
            return map;
        }

        private Dictionary<String, String> groupBeansAsMap(WSGroupBean[] beans)
        {
            Dictionary<String, String> map = new Dictionary<String, String>();
            if (beans != null)
            {
                WSGroupBean bean;
                for (int i = 0; i < beans.Length; i++)
                {
                    bean = beans[i];
                    map[bean.objectID] = bean.name;
                }
            }
            return map;
        }

        private void printRoleAssignments(WSRoleAssignmentBean[] roleAssignmentBeans, WSLabelValueBean[] allRoles, WSPersonBean[] allPersons,
                WSGroupBean[] allGroups)
        {
            if (roleAssignmentBeans == null || roleAssignmentBeans.Length == 0)
            {
                Console.WriteLine("No role assignment");
                return;
            }
            WSRoleAssignmentBean role;
            String projectID;
            String roleID;
            String roleName;
            String[] personIDs;
            String personID;
            String personName;
            Dictionary<String, String> roleMap = labelValueBeansAsMap(allRoles);
            Dictionary<String, String> personMap = personBeansAsMap(allPersons);
            Dictionary<String, String> groupMap = groupBeansAsMap(allGroups);

            for (int i = 0; i < roleAssignmentBeans.Length; i++)
            {
                role = roleAssignmentBeans[i];
                projectID = role.projectID;
                roleID = role.roleID;
                roleName = roleMap[roleID];
                personIDs = role.personIDs;
                Console.WriteLine("  roleID=" + roleID + " roleName=" + roleName);
                Console.WriteLine("  persons=[");
                for (int j = 0; j < personIDs.Length; j++)
                {
                    personID = personIDs[j];
                    personName = personMap[personID];
                    if (personName == null)
                    {
                        personName = "GROUP-" + groupMap[personID];
                    }
                    Console.WriteLine("{" + personID + ": " + personName + "} ");
                }
                Console.WriteLine("]\n");
            }
        }

        /*
         * Print the existing roles.
         */
        private void printRoles(WSLabelValueBean[] allRoles)
        {
            if (allRoles != null)
            {
                WSLabelValueBean role;
                for (int i = 0; i < allRoles.Length; i++)
                {
                    role = allRoles[i];
                    Console.WriteLine(i + " {name=" + role.label + "; ID=" + role.value + "}");
                }
            }
            else
            {
                Console.WriteLine("No roles");
            }
        }

        public static void runRoleSample(String[] args)
        {
            RoleSample roleSample = new RoleSample();

            // Login, the connection details are in parent class LoginSample
            roleSample.login(user, passwd);
            WSLabelValueBean[] allRoles = roleSample.getAllRoles();
            Console.WriteLine("All roles:");
            roleSample.printRoles(allRoles);

            GroupSample groupSample = new GroupSample(roleSample.connector);
            WSGroupBean[] allGroups = groupSample.getAllGroups();

            PersonSample personSample = new PersonSample(roleSample.connector);
            WSPersonBean[] allPersons = personSample.getAllUsers();

            String projectID = null;
            String projectName = null;

            ProjectSample projectSample = new ProjectSample(roleSample.connector);
            WSLabelValueBean[] allProjects = projectSample.getAllProjects();

            if (allProjects != null && allProjects.Length > 0)
            {
                WSRoleAssignmentBean[] roleAssignmentBeans;
                int i = 0;
                // for(i=0;i<allProjects.length;i++){
                projectID = allProjects[i].value;
                projectName = allProjects[i].label;
                Console.WriteLine("\nThe following listings are for project with projectID=" + projectID + ", projectName=" + projectName);
                roleAssignmentBeans = roleSample.getAssignmentRolesByProject(projectID);
                Console.WriteLine("\nRole assignments:");
                roleSample.printRoleAssignments(roleAssignmentBeans, allRoles, allPersons, allGroups);
                Console.WriteLine("");
                // }
            }

            if (projectID != null && allRoles != null && allRoles.Length > 0 && allPersons != null && allPersons.Length > 0)
            {
                // add role assignment
                String roleID = allRoles[0].value;
                String personID = allPersons[0].personID;
                String groupID = allGroups[0].objectID;
                WSRoleAssignmentEditBean wsRoleAssignmentEditBean = new WSRoleAssignmentEditBean();
                wsRoleAssignmentEditBean.projectID = projectID;
                wsRoleAssignmentEditBean.roleID = roleID;
                wsRoleAssignmentEditBean.personIDs = (new String[] { personID, groupID });

                roleSample.addRoleAssignment(wsRoleAssignmentEditBean);
                WSRoleAssignmentBean[] roleAssignmentBeans = roleSample.getAssignmentRolesByProject(projectID);
                Console.WriteLine("Role assignments after add:");
                roleSample.printRoleAssignments(roleAssignmentBeans, allRoles, allPersons, allGroups);

                // remove role
                roleSample.removeRoleAssignment(wsRoleAssignmentEditBean);

            }
        }

    }
}

